package hr.webtech.pay2.example;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Toast;

import ba.leftor.android.pikpay.BuildConfig;
import ba.leftor.android.pikpay.R;
import hr.webtech.pay2.api.PgwModule;
import hr.webtech.pay2.data.AddCardModel;
import hr.webtech.pay2.data.ApiError;
import hr.webtech.pay2.data.Customer;
import hr.webtech.pay2.data.Merchant;
import hr.webtech.pay2.data.OrderDetails;
import hr.webtech.pay2.data.PayCvvModel;
import hr.webtech.pay2.data.PayCvvOrderDetails;
import hr.webtech.pay2.data.PaymentMethod;
import hr.webtech.pay2.data.ShowOrderResponse;
import hr.webtech.pay2.data.TransactionSuccess;
import hr.webtech.pay2.example.data.MerchantProvider;
import hr.webtech.pay2.view.TransactionLifecycleAdapter;
import hr.webtech.pay2.view.TransactionLifecycleDelegate;
import hr.webtech.pay2.view.WebtehWebView;

/**
 * Example activity with {@link #webtehWebView}.
 * <p>
 * Use {@link #createIntent(Context, boolean)} to load example
 */
public class WebtehWebViewExampleActivity extends Activity {

    public static final String BNLD_PAY = "bnld_pay";
    public static final String BNLD_ADD_PAYMENT_METHOD = "BNLD_ADD_PAYMENT_METHOD";
    private static final String TAG = WebtehWebViewExampleActivity.class.getSimpleName();
    public static final String BNLD_MERCHANT = "bnld_merchant";
    public static final String BNLD_PAYMENT_METHOD = "bnld_payment_method";
    private View progressBar;
    private WebtehWebView webtehWebView;
    private Merchant merchant;
    private PaymentMethod paymentMethod;

    public static Intent createIntent(Context context, boolean payCvv) {
        return createIntent(context, payCvv, null);
    }

    public static Intent createIntent(Context context, boolean payCvv, Merchant merchant) {
        final Intent intent = new Intent(context, WebtehWebViewExampleActivity.class);
        intent.putExtra(BNLD_PAY, payCvv);
        intent.putExtra(BNLD_MERCHANT, merchant);
        return intent;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_webteh_web_view_example);

        final Intent intent = getIntent();

        final Merchant merchantParcelable = intent.getParcelableExtra(BNLD_MERCHANT);

        if (merchantParcelable == null) {
            this.merchant = MerchantProvider.getAllMerchants().get(0);
        } else {
            this.merchant = merchantParcelable;
        }

        this.paymentMethod = intent.getParcelableExtra(BNLD_PAYMENT_METHOD);

        this.webtehWebView = (WebtehWebView) findViewById(R.id.webView);
        this.progressBar = findViewById(R.id.progressBar);
        final boolean isPay = intent.getBooleanExtra(BNLD_PAY, false);

        PgwModule pgwModule = new PgwModule(this.merchant.getAuthenticationToken(), BuildConfig.DEBUG);

        final String orderNumber = String.valueOf(System.currentTimeMillis());

        final TransactionLifecycleDelegate transactionLifecycleDelegate = new TransactionLifecycleDelegateImpl(getApplicationContext());
        final TransactionLifecycleAdapter.ResponseType responseType = TransactionLifecycleAdapter.ResponseType.TransactionSuccess;

        final TransactionLifecycleAdapter transactionLifecycleAdapter = new TransactionLifecycleAdapter(
                orderNumber,
                pgwModule.getApi(),
                this.merchant.getMerchantKey(),
                responseType,
                transactionLifecycleDelegate
        );


        Log.d(TAG, "onCreate: " + getApplicationContext().getResources().getConfiguration().locale);

        if (!isPay) {
            addCardExample(transactionLifecycleAdapter, orderNumber);
        } else {
            payCvvExample(transactionLifecycleAdapter, orderNumber);
        }
    }

    private void payCvvExample(TransactionLifecycleAdapter transactionLifecycleAdapter, String orderNumber) {

        OrderDetails orderDetails = new OrderDetails(
                "1250",
                orderNumber,
                AddCardModel.Currency.EUR.toString(),
                "order info",
                "009",
                "cvv_android"
        );

        Customer customer = new Customer(
                "John Smith",
                "test@test.com",
                "Address",
                "+1-202-555-0138",
                "Sarajevo",
                "test@test.com"
        );

        PayCvvOrderDetails payCvvOrderDetails = new PayCvvOrderDetails(orderDetails, this.paymentMethod.getId());

        PayCvvModel payCvvModel = new PayCvvModel(merchant, payCvvOrderDetails, customer);
        webtehWebView
                .initialize(BuildConfig.DEBUG, 60000)
                .payCvv(transactionLifecycleAdapter, payCvvModel);
    }

    private void addCardExample(TransactionLifecycleAdapter transactionLifecycleAdapter, String orderNumber) {

        Customer customer = new Customer(
                "John Smith",
                "test@test.com",
                "Address",
                "+1-202-555-0138",
                "Sarajevo",
                "test@test.com"
        );

        OrderDetails orderDetails = new OrderDetails(
                "1250",
                orderNumber,
                AddCardModel.Currency.EUR.toString(),
                "order info",
                "009",
                "android"
        );

        webtehWebView
                .initialize(BuildConfig.DEBUG, 60000)
                .addCard(transactionLifecycleAdapter, new AddCardModel(merchant, orderDetails, customer));
    }

    /**
     * To prevent stopping ongoing transaction add code bellow
     */
    @Override
    public void onBackPressed() {
        if (webtehWebView.isBackAllowed()) {
            super.onBackPressed();
        }
    }

    /**
     * For best behaviour invoke {@link WebtehWebView#onDestroy()} in appropriate lifecycle event
     */
    @Override
    protected void onDestroy() {
        super.onDestroy();
        webtehWebView.onDestroy();
    }

    public static Intent createIntent(Context context, boolean payCvv, Merchant merchant, PaymentMethod paymentMethod) {
        final Intent intent = new Intent(context, WebtehWebViewExampleActivity.class);
        intent.putExtra(BNLD_PAY, payCvv);
        intent.putExtra(BNLD_MERCHANT, merchant);
        intent.putExtra(BNLD_PAYMENT_METHOD, paymentMethod);
        return intent;
    }

    final class TransactionLifecycleDelegateImpl implements TransactionLifecycleDelegate {

        private final Context context;

        TransactionLifecycleDelegateImpl(Context context) {
            this.context = context;
        }

        @Override
        public void showProgress() {
            progressBar.setVisibility(View.VISIBLE);
        }

        @Override
        public void hideProgress() {
            progressBar.setVisibility(View.INVISIBLE);
        }

        @Override
        public void transactionSuccess(TransactionSuccess transaction) {
            Log.d(TAG, "transactionSuccess: " + transaction);
            Toast.makeText(context, transaction.toString(), Toast.LENGTH_LONG).show();
        }

        @Override
        public void backPressedTransactionInProgress() {
            Log.d(TAG, "backPressedTransactionInProgress");
        }

        @Override
        public void threeDsSubmitted() {
            Log.d(TAG, "threeDsSubmitted");
        }

        @Override
        public void threeDsLoaded() {
            Log.d(TAG, "threeDsLoaded");
        }

        @Override
        public void handleTransactionError(ApiError apiError) {
            Log.e(TAG, "handleTransactionError: " + apiError, apiError);
        }

        @Override
        public void checkOrderIdSuccess(ShowOrderResponse showOrderResponse) {
            Log.d(TAG, "checkOrderIdSuccess: " + showOrderResponse);
        }

        @Override
        public void stopWebViewLoad() {
            Log.d(TAG, "stopWebViewLoad");
        }

        @Override
        public void logTransactionInfo(String loggingInfo) {
            Log.d(TAG, "logTransactionInfo: " + loggingInfo);
        }
    }

}
