
package hr.webtech.pay2.data;

import android.os.Parcel;
import android.os.Parcelable;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;

import com.google.gson.annotations.Expose;
import com.google.gson.annotations.SerializedName;

import hr.webtech.pay2.util.Objects;
import hr.webtech.pay2.util.Utils;
import hr.webtech.pay2.view.WebtehWebView;


/**
 * AddCard
 * <p>
 * Add card model for {@link WebtehWebView#addCard(AddCardModel)}
 * <p>
 * Check <a href='https://gist.github.com/jasminsuljic/d1f0702371e16c91f5d6785826ce4ecb'json schema></a>
 */
public final class AddCardModel implements Parcelable {

    @SerializedName("client")
    @Expose
    String client;
    /**
     * (Required)
     */
    @SerializedName("merchant_id")
    @Expose
    String merchantId;
    /**
     * short description of order being processed
     */
    @SerializedName("order_info")
    @Expose
    String orderInfo;
    /**
     * unique identifier
     * (Required)
     */
    @SerializedName("order_number")
    @Expose
    String orderNumber;
    @SerializedName("sign")
    @Expose
    String sign;
    /**
     * amount is in minor units, ie. 10.24 USD is sent as 1024
     * (Required)
     */
    @SerializedName("amount")
    @Expose
    String amount;
    /**
     * (Required)
     */
    @SerializedName("currency")
    @Expose
    String currency;
    /**
     * (Required)
     */
    @SerializedName("client_secret")
    @Expose
    String clientSecret;
    /**
     * (Required)
     */
    @SerializedName("auth_token")
    @Expose
    String authToken;

    @SerializedName("digest")
    @Expose
    String digest;


    @SerializedName("address")
    @Expose
    @Nullable
    String address;

    @SerializedName("phone_number")
    @Expose
    @Nullable
    String phoneNumber;

    @SerializedName("city")
    @Expose
    @Nullable
    String city;

    @SerializedName("email")
    @Expose
    @Nullable
    String email;

    @SerializedName("card_holder")
    @Expose
    @Nullable
    String cardHolder;

    @SerializedName("user_id")
    @Expose
    @Nullable
    String userId;


    @Deprecated
    public AddCardModel(String amount,
                        String authToken,
                        String clientSecret,
                        String merchantId,
                        String orderNumber,
                        String sign,
                        String currency,
                        String orderInfo,
                        String client,
                        @Nullable String address,
                        @Nullable String phoneNumber,
                        @Nullable String city,
                        @Nullable String email,
                        @Nullable String cardHolder,
                        @Nullable String userId) {
        this.amount = amount;
        this.authToken = authToken;
        this.clientSecret = clientSecret;
        this.merchantId = merchantId;
        this.orderNumber = orderNumber;
        this.sign = sign;
        this.currency = currency;
        this.orderInfo = orderInfo;
        this.client = client;

        this.address = address;
        this.phoneNumber = phoneNumber;
        this.city = city;

        this.email = email;
        this.cardHolder = cardHolder;

        this.userId = userId;

        generateDigest();
    }

    private AddCardModel(Parcel in) {
        merchantId = in.readString();
        orderInfo = in.readString();
        orderNumber = in.readString();
        sign = in.readString();
        amount = in.readString();
        currency = in.readString();
        clientSecret = in.readString();
        authToken = in.readString();
        client = in.readString();
        address = in.readString();
        phoneNumber = in.readString();
        city = in.readString();
        email = in.readString();
        cardHolder = in.readString();
        userId = in.readString();

        generateDigest();
    }

    public AddCardModel(@NonNull Merchant merchant,
                        @NonNull OrderDetails orderDetails,
                        @Nullable Customer customer) {

        Objects.requireNonNull(merchant);
        Objects.requireNonNull(orderDetails);

        this.amount = orderDetails.getAmount();
        this.authToken = merchant.getAuthenticationToken();
        this.clientSecret = merchant.getClientSecret();
        this.merchantId = merchant.getMerchantId();
        this.orderNumber = orderDetails.getOrderNumber();
        this.sign = orderDetails.getSignature();
        this.currency = orderDetails.getCurrency();
        this.orderInfo = orderDetails.getOrderInfo();
        this.client = orderDetails.getClient();

        if (customer == null) {
            this.address = null;
            this.phoneNumber = null;
            this.city = null;
            this.email = null;
            this.cardHolder = null;
            this.userId = null;
        } else {
            this.address = customer.getAddress();
            this.phoneNumber = customer.getPhoneNumber();
            this.city = customer.getCity();
            this.email = customer.getEmail();
            this.cardHolder = customer.getCardHolder();
            this.userId = customer.getUserId();
        }

        this.generateDigest();
    }

    private void generateDigest() {

        String sb1 = merchantId +
                authToken +
                orderNumber +
                sign +
                amount +
                currency +
                clientSecret;

        this.digest = Utils.sha1Hash(sb1);
    }

    public static final Creator<AddCardModel> CREATOR = new Creator<AddCardModel>() {
        @Override
        public AddCardModel createFromParcel(Parcel in) {
            return new AddCardModel(in);
        }

        @Override
        public AddCardModel[] newArray(int size) {
            return new AddCardModel[size];
        }
    };

    @Override
    public int describeContents() {
        return 0;
    }

    @Override
    public void writeToParcel(Parcel dest, int flags) {

        dest.writeString(merchantId);
        dest.writeString(orderInfo);
        dest.writeString(orderNumber);
        dest.writeString(sign);
        dest.writeString(amount);
        dest.writeString(currency);
        dest.writeString(clientSecret);
        dest.writeString(authToken);
        dest.writeString(client);
        dest.writeString(address);
        dest.writeString(phoneNumber);
        dest.writeString(city);
        dest.writeString(email);
        dest.writeString(cardHolder);
        dest.writeString(userId);
    }

    public enum Currency {

        @SerializedName("USD")
        USD("USD"),
        @SerializedName("EUR")
        EUR("EUR"),
        @SerializedName("BAM")
        BAM("BAM"),
        @SerializedName("HRK")
        HRK("HRK");
        public final String value;

        Currency(String value) {
            this.value = value;
        }

        @Override
        public String toString() {
            return this.value;
        }

    }

    public String getOrderNumber() {
        return orderNumber;
    }
}