package hr.webtech.pay2.data;

import android.support.annotation.NonNull;
import android.support.annotation.Nullable;

import com.google.gson.annotations.Expose;
import com.google.gson.annotations.SerializedName;

/**
 * Created by jasminsuljic on 24/01/17.
 * pgw-android
 */

public class ApiError extends Throwable implements ValidityInterface {

    @Expose
    @SerializedName("code")
    int code;

    @SerializedName("error")
    @Expose
    @Nullable
    String error;

    /**
     * True when error is generated from library/api. False if error is generated on payment gateway.
     * <p>
     * Alias: library_error
     */
    @SerializedName("library_error")
    @Expose
    boolean applicationError;

    /**
     * True when transaction definitely failed and user was not charged
     */
    @SerializedName("definite_error")
    @Expose
    boolean definiteError;

    /**
     * Error log id generated after error occurs.
     */
    @SerializedName("error_id")
    @Expose
    int errorId = -1;

    ApiError(int code, boolean applicationError, @NonNull Throwable cause) {
        super(cause);
        this.code = code;
        this.applicationError = applicationError;
        this.error = cause.getMessage();
    }

    public ApiError(boolean applicationError, int code, boolean definiteError, @Nullable String error, int errorId) {
        super(error);
        this.applicationError = applicationError;
        this.code = code;
        this.definiteError = definiteError;
        this.error = error;
        this.errorId = errorId;
    }

    public int getCode() {
        return code;
    }

    public boolean isApplicationError() {
        return applicationError;
    }

    public boolean isBusinessError() {
        return !applicationError;
    }

    public boolean isDefiniteError() {
        return definiteError;
    }

    public int getErrorId() {
        return errorId;
    }

    @Nullable
    public String getError() {
        return error;
    }

    @Override
    public String toString() {
        return "ApiError{" +
                "applicationError=" + applicationError +
                ", code=" + code +
                ", error='" + error + '\'' +
                ", definiteError=" + definiteError +
                ", errorId=" + errorId +
                '}';
    }

    @Override
    public boolean isValid() {
        return true;
    }
}
