package hr.webtech.pay2.data;

import android.support.annotation.NonNull;

import com.google.gson.annotations.Expose;
import com.google.gson.annotations.SerializedName;

/**
 * Created by jasminsuljic on 07/02/17.
 * pgw-android
 */

public final class BusinessError extends ApiError {

    @Expose
    @SerializedName("transaction_id")
    int transactionId = -1;

    @Expose
    @SerializedName("masked_pan")
    String maskedPan;

    @Expose
    @SerializedName("full_name")
    String fullName;

    @Expose
    @SerializedName("card_type")
    String cardType;

    @Expose
    @SerializedName("pm_alias")
    String pmAlias;

    public BusinessError(int code, boolean applicationError, @NonNull Throwable cause) {
        super(code, applicationError, cause);
    }

    public String getCardType() {
        return cardType;
    }

    public String getFullName() {
        return fullName;
    }

    public String getMaskedPan() {
        return maskedPan;
    }

    public String getPmAlias() {
        return pmAlias;
    }

    public int getTransactionId() {
        return transactionId;
    }

    @Override
    public String toString() {
        return "BusinessError{" +
                "applicationError=" + applicationError +
                ", code=" + code +
                ", error='" + error + '\'' +
                ", definiteError=" + definiteError +
                ", errorId=" + errorId +
                ", cardType='" + cardType + '\'' +
                ", transactionId=" + transactionId +
                ", maskedPan='" + maskedPan + '\'' +
                ", fullName='" + fullName + '\'' +
                ", pmAlias='" + pmAlias + '\'' +
                '}';
    }

    /**
     * Returns true if: transactionId, maskedPan, fullNae, cardType is set and if error is business error
     *
     * @return
     */
    @Override
    public boolean isValid() {
        return transactionId != -1 && maskedPan != null && fullName != null && cardType != null && isBusinessError();
    }

    public final ApiError asApiError(){
        return new ApiError(this.applicationError, this.code, this.definiteError, this.error, this.errorId);
    }
}
